/**
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit.gui;

/** La classe Compteur qui fabrique un compteur hh:mm:ss:centieme de s* */
public class CounterCalculation
{
	private int heure;
	private int minute;
	private int seconde;
	private int centiSeconde;

	// cons. par defaut
	public CounterCalculation()
	{
		heure = 0;
		minute = 0;
		seconde = 0;
		centiSeconde = 0;
	}

	// avec initialisation
	public CounterCalculation(int h, int m, int s, int cs)
	{
		heure = h;
		minute = m;
		seconde = s;
		centiSeconde = cs;
	}

	/** methode pour modifier la date */
	public void setValeur(int H, int M, int S, int Cs)
	{
		while (Cs >= 100)
		{
			Cs = Cs - 100;
			S = S + 1;
		}
		while (S >= 60)
		{
			S = S - 60;
			M = M + 1;
		}
		while (M >= 60)
		{
			M = M - 60;
			H = H + 1;
		}
		heure = H;
		minute = M;
		seconde = S;
		centiSeconde = Cs;
	}

	/** methode pour modifier la date */
	public void setValeurDelta(int S, int Cs)
	{
		while (Cs >= 100)
		{
			Cs = Cs - 100;
			S = S + 1;
		}
		heure = 0;
		minute = 0;
		seconde = S;
		centiSeconde = Cs;
	}

	/** methode de remise a zero du compteur */
	public void raz()
	{
		heure = 0;
		minute = 0;
		seconde = 0;
		centiSeconde = 0;
	}

	/** methodes d'incrementation */
	public void addHeures(double absTime, int val)
	{
		double absNewTime = absTime + val * 360000;
		if (absNewTime < 0)
			absNewTime = 0;
		toCompteur(absNewTime);
	}

	public void addMinutes(double absTime, int val)
	{
		double absNewTime = absTime + val * 6000;
		if (absNewTime < 0)
			absNewTime = 0;
		toCompteur(absNewTime);
	}

	public void addSecondes(double absTime, int val)
	{
		double absNewTime = absTime + val * 100;
		if (absNewTime < 0)
			absNewTime = 0;
		toCompteur(absNewTime);
	}

	public void addSecondesD(double absTime, int val)
	{
		double absNewTime = absTime + val * 100;
		if (absNewTime < 10)
			absNewTime = 10; // valeur min pour delta : 10 centisec
		if (absNewTime > 6000)
			absNewTime = 6000;
		toCompteur(absNewTime);
	}

	public void addCentiSecondes(int absTime, int val)
	{
		int absNewTime = absTime + val;
		if (absNewTime < 0)
			absNewTime = 0;
		toCompteur(absNewTime);
	}

	/** pour recuperer la valeur du compteur */
	public int getValHeure()
	{
		return heure;
	}

	public int getValMinute()
	{
		return minute;
	}

	public int getValSeconde()
	{
		return seconde;
	}

	public int getValCentiSeconde()
	{
		return centiSeconde;
	}

	/** passage du temps compteur en milliseconde */
	public int toCentiSeconde()
	{
		return (centiSeconde + 100 * (seconde + 60 * (minute + 60 * (heure))));
	}

	public int toMilliSeconde()
	{
		return (10 * centiSeconde + 1000 * (seconde + 60 * (minute + 60 * (heure))));
	}

	/** passage du temps en milliseconde en temps compteur */
	public boolean toCompteur(double temps)
	{
		if(temps != toCentiSeconde())
		{
			raz();
			// if (temps < 0) temps = 0 ;
			heure = (int) (temps / 360000);
			minute = (int) ((temps - (heure * 360000)) / 6000);
			seconde = (int) ((temps - (heure * 360000) - (minute * 6000)) / 100);
			centiSeconde = (int) (temps - (heure * 360000) - (minute * 6000) - (seconde * 100));
			return true;
		}
		return false;
	}

	/** soustraction de temps, retourne un temp en milliseconde */
	public CounterCalculation soustract(CounterCalculation c)
	{
		CounterCalculation res = new CounterCalculation();
		double temps1 = this.toCentiSeconde();
		double temps2 = c.toCentiSeconde();
		res.toCompteur(temps2 - temps1);
		return res;
	}

	/** addition de temps, retourne un temp en milliseconde */
	public CounterCalculation addition(CounterCalculation c)
	{
		CounterCalculation res = new CounterCalculation();
		double temps1 = this.toCentiSeconde();
		double temps2 = c.toCentiSeconde();
		res.toCompteur(temps2 + temps1);
		return res;
	}

	/** comparaison de deux compteur*/
	public boolean inferieur(CounterCalculation c)
	{
		if (this.soustract(c).toCentiSeconde() < 0)
			return false;
		return true;
	}
}