/**
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit;

import holoedit.data.HoloTrack;
import holoedit.data.HoloPoint;
import holoedit.gui.GestionPistes;
import holoedit.util.Ut;
import java.awt.Color;
import java.io.File;
import java.util.Vector;
import javax.swing.SwingUtilities;
import com.cycling74.max.Atom;
import com.cycling74.max.MaxClock;
import com.cycling74.max.MaxSystem;

public class TjPlayer extends HoloPlayer
{
	private static double version = 0.2;
	private boolean cuemode = true;
	private float speed  = 1.f;
	private boolean share = false;
	private int lastcue = 1;
	private float fcounter = 0.f;
	private static GestionPistes shared_gp;
	private Vector<String> cue = new Vector<String>();
	private static Vector<String> shared_cue = new Vector<String>();
	
	protected void initObj()
	{
		// On recupere le nom du system d'exploitation et on affecte les
		// variables en fonction
		Ut.MAC = System.getProperty("os.name").indexOf("Mac") >= 0;
		if (Ut.MAC)
		{
			Ut.dirCar = "/";
		} else {
			Ut.dirCar = "\\";
		}
		
		clock = new MaxClock(this);
		createInfoOutlet(false);
		
		declareAttribute("cuemode","isCuemode","setCuemode");
		declareAttribute("loop","isLoop","setLoop");
		declareAttribute("speed","getSpeed","setSpeed");
		declareAttribute("share","isShare","setShare");
		
		if(share)
		{
			if(shared_gp == null)
				shared_gp = new GestionPistes(this,true);
		} else {
			gestionPistes = new GestionPistes(this,true);
		}
	}

	public void version()
	{
		post("---------------------------------");
		post("");
		post("\t\tHolo-Edit trajectory player v"+Ut.version);
		post("");
		post("---------------------------------");
	}
	
	public void read()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				if(share)
				{
					shared_gp.initTracks2(true);
					shared_gp.setActiveTrack(0);
					shared_gp.importSeq(0, 0);
					shared_cue.clear();
					shared_cue.add(session);
				} else {
					gestionPistes.initTracks2(true);
					gestionPistes.setActiveTrack(0);
					gestionPistes.importSeq(0, 0);
					cue.clear();
					cue.add(session);
				}
			}
		});
	}
	
	public void read(String fn)
	{
		session = fn;
		File f = new File(session);
		if(!f.exists())
			session = MaxSystem.locateFile(session);
		if(share)
		{
			if(shared_gp.setTjFileInName(session))
			{
				SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						shared_gp.initTracks2(true);
						shared_gp.setActiveTrack(0);
						shared_gp.importSeq(0, 0);
						shared_cue.clear();
						shared_cue.add(session);
					}
				});
			}
		} else {
			if(gestionPistes.setTjFileInName(session))
			{
				SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						gestionPistes.initTracks2(true);
						gestionPistes.setActiveTrack(0);
						gestionPistes.importSeq(0, 0);
						cue.clear();
						cue.add(session);
					}
				});
			}
		}
	}
	
	public void init()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				if(share)
				{
					shared_gp.initTracks2(true);
					shared_gp.setActiveTrack(0);
					shared_cue.clear();
				} else {
					gestionPistes.initTracks2(true);
					gestionPistes.setActiveTrack(0);
					cue.clear();
				}
			}
		});
	}
	
	public void preload(Atom[] args)
	{
		if(args.length >= 2)
		{
			int tocue = args[0].toInt() - 1;
			session = args[1].toString();
			File f = new File(session);
			if(!f.exists())
				session = MaxSystem.locateFile(session);
			if(share)
			{
				if(shared_gp.setTjFileInName(session))
				{
					System.out.println("Import "+session+" to cue/tk "+(tocue+1));
					if(tocue >= shared_gp.getNbTracks())
						for(int i = shared_gp.getNbTracks() ; i <= tocue ; i++)
							shared_gp.tracks.add(new HoloTrack(i+1, Color.black, true));
					if(tocue >= shared_cue.size())
						for(int i = shared_cue.size() ; i <= tocue ; i++)
							shared_cue.add("none");
					
					shared_gp.setActiveTrack(tocue);
					shared_gp.importSeq2(tocue, 0);
					shared_cue.set(tocue, session);
				}
			} else {
				if(gestionPistes.setTjFileInName(session))
				{
					System.out.println("Import "+session+" to cue/tk "+(tocue+1));
					if(tocue >= gestionPistes.getNbTracks())
						for(int i = gestionPistes.getNbTracks() ; i <= tocue ; i++)
							gestionPistes.tracks.add(new HoloTrack(i+1, Color.black, true));
					if(tocue >= cue.size())
						for(int i = cue.size() ; i <= tocue ; i++)
							cue.add("none");
					
					gestionPistes.setActiveTrack(tocue);
					gestionPistes.importSeq2(tocue, 0);
					cue.set(tocue, session);
				}
			}
		}
	}
	
	public void printcue()
	{
		if(share)
		{
			post("current trajectory cue : ("+shared_cue.size()+" trajectories)");
			for(int i = 0 ; i < shared_cue.size() ; i++)
				post("   "+(i+1)+" "+shared_cue.get(i));
		} else {
			post("current trajectory cue : ("+cue.size()+" trajectories)");
			for(int i = 0 ; i < cue.size() ; i++)
				post("   "+(i+1)+" "+cue.get(i));
		}
	}
	
	public void stop()
	{
		super.stop();
		send("stop");
	}
	
	public void inlet(int i)
	{
		if(i <= 0)
			stop();
		else if(!cuemode)
			play();
		else if((share && i <= shared_cue.size()) || i <= cue.size()) {
			lastcue = i-1;
			play();
		}
	}
	
	public void play()
	{
		autostop = false;
		playing = true;
		firstCue = true;
		loopNum = 0;
		fcounter = counter = begtime;
		if(cuemode)
		{
			endtime = share ? shared_gp.getTrack(lastcue).getLastDate() : gestionPistes.getTrack(lastcue).getLastDate();
		} else {
			if(share)
			{
				endtime = 0;
				for(int i = 0 ; i < shared_gp.getNbTracks() ; i++)
					endtime = Ut.max(endtime, shared_gp.getTrack(i).getLastDate());
			} else {
				endtime = 0;
				for(int i = 0 ; i < gestionPistes.getNbTracks() ; i++)
					endtime = Ut.max(endtime, gestionPistes.getTrack(i).getLastDate());
			}
		}
		looping = !firstCue;
		clock.delay(0);
	}
	
	public void execute()
	{
		if(!paused && playing)
		{
			if(counter > endtime)
			{
				if(loop)
				{
					send("looping");
					fcounter = counter = begtime;
					looping = true;
					loopNum++;
				} else {
					send("stop");
					playing = false;
					paused = false;
					autostop = true;
				}
			}

			send("time "+counter);
			
			if(share)
			{
				if (cuemode)
				{
					send("cue " + (lastcue + 1));
					currentTrack = shared_gp.getTrack(lastcue);
					String vc1 = currentTrack.getValueAt(counter, autostop, looping);
					if (!vc1.equalsIgnoreCase("none"))
						send("pos " + vc1);
				}
				else
				{
					send("cue " + -1);
					for (int i = 0, last = shared_gp.getNbTracks(); i < last; i++)
					{
						currentTrack = shared_gp.getTrack(i);
						if (currentTrack.isVisible())
						{
							String vc1 = currentTrack.getValueAt(counter, autostop, looping);
							if (!vc1.equalsIgnoreCase("none"))
								send("tk " + currentTrack.getNumber() + " pos " + vc1);
						}
					}
				}
			}
			else
			{
				if (cuemode)
				{
					send("cue " + (lastcue + 1));
					currentTrack = gestionPistes.getTrack(lastcue);
					String vc1 = currentTrack.getValueAt(counter, autostop, looping);
					if (!vc1.equalsIgnoreCase("none"))
						send("pos " + vc1);
				}
				else
				{
					send("cue " + -1);
					for (int i = 0, last = gestionPistes.getNbTracks(); i < last; i++)
					{
						currentTrack = gestionPistes.getTrack(i);
						if (currentTrack.isVisible())
						{
							String vc1 = currentTrack.getValueAt(counter, autostop, looping);
							if (!vc1.equalsIgnoreCase("none"))
								send("tk " + currentTrack.getNumber() + " pos " + vc1);
						}
					}
				}
			}
			
			looping = false;
			fcounter += speed;
			counter	= (int) (fcounter + 0.5f);
			if(!autostop)
				clock.delay(tick);
		}
	}
	
	public void write(){}
	
	public void writeagain(){}
	
	public void preload(){}
	
	public void update(){}
	
	public void queryspeakers(){}
	
	public void record(int i){}

	private boolean isCuemode()
	{
		return cuemode;
	}

	private void setCuemode(boolean cuemode)
	{
//		System.out.println("cuemode : "+cuemode);
		stop();
		this.cuemode = cuemode;
	}
	
	protected boolean isLoop()
	{
		return super.isLoop();
	}
	
	protected void setLoop(boolean loop)
	{
//		System.out.println("loop : "+loop);
		super.setLoop(loop);
	}
	
	public void dump()
	{
		for(int i = 0 ; i < cue.size() ; i++)
			dump(i+1);
	}
	
	public void dump(int i)
	{
		if(share)
		{
			int currentCue;
			if (i > shared_cue.size() || i < 1)
			{
				post("TjPlayer : no cue " + i + " to dump");
				return;
			}
			currentCue = i - 1;
			for (HoloPoint p : shared_gp.getTrack(currentCue).elements())
				send("dump " + i + " " + p.date + " " + p.x + " " + p.y + " " + p.z + " " + p.isEditable());
		}
		else
		{
			int currentCue;
			if (i > cue.size() || i < 1)
			{
				post("TjPlayer : no cue " + i + " to dump");
				return;
			}
			currentCue = i - 1;
			for (HoloPoint p : gestionPistes.getTrack(currentCue).elements())
				send("dump " + i + " " + p.date + " " + p.x + " " + p.y + " " + p.z + " " + p.isEditable());
		}
	}
	
	private void setSpeed(float speed)
	{
//		System.out.println("speed : "+speed);
		this.speed = (float)Math.min(Math.max(speed,0.),64.);
	}
	
	private float getSpeed()
	{
		return speed;
	}

	private boolean isShare()
	{
		return share;
	}

	private void setShare(boolean share)
	{
//		System.out.println("share : "+share);
		stop();
		this.share = share;
		if(share)
		{
			gestionPistes = null;
			cue.clear();
			if(shared_gp == null)
			{
				shared_gp = new GestionPistes(this,true);
				shared_cue.clear();
			}
		} else {
			gestionPistes = new GestionPistes(this,true);
			cue.clear();
		}
	}
}